using System;
using System.IO;
using System.Text;
using System.Runtime.InteropServices;
using System.Diagnostics;

namespace USB_HID
{

    public enum OutputValue
    {
        REPORT_ID_CMD = 0xf0,
        NavReport = 0x01,
        Reload = 0x02,
        Led = 0x04,
        Init=0x06,
        Calibrate = 0x07,
        Profil = 0x08,
        Config = 0xFF
    }

    public class NavStates : HIDStates
    {
        public NavStates()
        {
            base.HitInit(21, 6);
        }
    
    }

    #region Event definitions

    public class HIDChangedEventArgs : EventArgs
    {

        public readonly NavStates HIDValue;
        public HIDChangedEventArgs(NavStates arrStates)
        {
            HIDValue = arrStates;
        }
    }

    public delegate void HIDChangedEventHandler(object sender, HIDChangedEventArgs args);

    #endregion

    public class SpaceNavigator : HIDDevice
    {
        #region Public attributes/methods


        public event HIDChangedEventHandler OnButtonChanged;
        public event HIDChangedEventHandler OnDeviceRemoved;

        public override InputReport CreateInputReport()
        {
            return new BuzzInputReport(this);
        }
        private NavStates CurrentHidValue;

        public void SetLed(bool bLight)
        {
            
            if (!CurrentHidValue.IsRBC9Present) return;

            BuzzOutputReport oRep = new BuzzOutputReport(this);
            oRep.SetOutput(OutputValue.Led, bLight);

            WriteOutput(oRep);
        }


        public void SetModeConfig(bool SendOn)
        {
            if (!CurrentHidValue.IsRBC9Present) return;

            BuzzOutputReport oRep = new BuzzOutputReport(this);
            oRep.SetOutput(OutputValue.Config, SendOn);

            WriteOutput(oRep);
        }

        public void ReloadProfil()
        {
            BuzzOutputReport oRep = new BuzzOutputReport(this);
            oRep.SetOutput(OutputValue.Reload);

            WriteOutput(oRep);
        }

        public void GetNavReport()
        {
 
            BuzzOutputReport oRep = new BuzzOutputReport(this);
            oRep.SetOutput(OutputValue.NavReport);

            WriteOutput(oRep);
        }

        public void Calibrate()
        {
            BuzzOutputReport oRep = new BuzzOutputReport(this);
            oRep.SetOutput(OutputValue.Calibrate);

            WriteOutput(oRep);
        }

        public void SelectProfil(short ProfilNum)
        {
            //Debug.Write(CurrentHidValue.IsRBC9Present);
            if (!CurrentHidValue.IsRBC9Present) return;

            BuzzOutputReport oRep = new BuzzOutputReport(this);
            //oRep.SetOutput(OutputValue.Profil, ProfilNum);
            //Debug.Write(OutputValue.Profil);
            oRep.SetOutput(OutputValue.Profil, ProfilNum);

            WriteOutput(oRep);

        }

        private void WriteOutput(BuzzOutputReport oRep)
        {
            try
            {
                //Debug.WriteLine("here");
                Write(oRep); // write the output report
            }
            catch (HIDDeviceException)
            {
                Console.WriteLine("Erreur 9");
                Debug.WriteLine("Erreur 9");
                // Device may have been removed!
            }
        }


        public SpaceNavigator()
        {

            CurrentHidValue = new NavStates();
            //Debug.Write(CurrentHidValue.IsRBC9Present);
        }

        public static SpaceNavigator FindSpaceNav(int nPid, short nUsage)
        {
            return  (SpaceNavigator)FindDevice(0x046D, nPid, nUsage, typeof(SpaceNavigator));
        }


        private void Parse_TrameHID(byte[] TrameHID)
        {

            switch (TrameHID[0])
            {
                case 1:

                    CurrentHidValue.Axe[0].State_Current = (short)(TrameHID[1] + (TrameHID[2] << 8));
                    CurrentHidValue.Axe[1].State_Current = (short)(TrameHID[3] + (TrameHID[4] << 8));
                    CurrentHidValue.Axe[2].State_Current = (short)(TrameHID[5] + (TrameHID[6] << 8));
                    break;

                case 2:

                    CurrentHidValue.Axe[3].State_Current = (short)(TrameHID[1] + (TrameHID[2] << 8));
                    CurrentHidValue.Axe[4].State_Current = (short)((TrameHID[3] + (TrameHID[4] << 8)) * -1);
                    CurrentHidValue.Axe[5].State_Current = (short)(TrameHID[5] + (TrameHID[6] << 8));
                    break;

                case 3:
                 
                    for (int BclBtn = 0; BclBtn < CurrentHidValue.NbBtn; BclBtn++)
                    {
                        byte BitMask = (byte) (1 << (BclBtn % 8));
                        CurrentHidValue.Btn[BclBtn].State_Current = (TrameHID[(BclBtn / 8)+1] & BitMask) == BitMask;
                    }
 
                    break;

                case (byte) OutputValue.REPORT_ID_CMD:
                    CurrentHidValue.ProfilNum = (short)(TrameHID[2] + (TrameHID[3] << 8));
                    CurrentHidValue.IsRBC9Present = ((short)(TrameHID[1]) != 0) ? true : false;
                    //CurrentHidValue.IsRBC9Present = true;
                    break;

            }
        }

        #endregion

        #region Overrides

        protected override void HandleDataReceived(InputReport oInRep)
        {
            BuzzInputReport oBuzIn = (BuzzInputReport)oInRep;
            Parse_TrameHID(oBuzIn.TrameHID);
            if (OnButtonChanged != null)
            {
                OnButtonChanged(this, new HIDChangedEventArgs(CurrentHidValue));
            }
        }

        protected override void HandleDeviceRemoved()
        {

            if (OnDeviceRemoved != null)
            {
                OnDeviceRemoved(this, new HIDChangedEventArgs(CurrentHidValue));
            }

        }


        protected override void Dispose(bool bDisposing)
        {
            if (bDisposing)
            {
                // before we go, turn all lights off
                //SetLights(0);
            }
            base.Dispose(bDisposing);
        }
        #endregion
    }

    #region Device reports

    public class BuzzOutputReport : OutputReport
    {
        public BuzzOutputReport(HIDDevice oDev) : base(oDev) { }
       
        public void SetOutput(OutputValue Command, bool bLight)
        {
            byte[] arrBuff = Buffer;

            //Debug.WriteLine("here2");
            arrBuff[0] = (byte) OutputValue.REPORT_ID_CMD;
            arrBuff[1] = (byte) Command;
            arrBuff[2] = (byte) (0x0001 & (bLight ? 1 : 0));
            //arrBuff[2] = (byte)(0x0002);
        }

        public void SetOutput(OutputValue Command)
        {
            byte[] arrBuff = Buffer;

            arrBuff[0] = (byte) OutputValue.REPORT_ID_CMD;
            arrBuff[1] = (byte) Command;
        }

        public void SetOutput(OutputValue Command, short NumProfil)
        {

            byte[] arrBuff = Buffer;

            arrBuff[0] = (byte) OutputValue.REPORT_ID_CMD;
            arrBuff[1] = (byte) Command;
            arrBuff[2] = (byte)(NumProfil & 0x00ff);
            arrBuff[3] = (byte)((NumProfil & 0xff00) >> 8);
        }
    }

    public class BuzzInputReport : InputReport
    {
        private byte[] m_TrameHID;
        public BuzzInputReport(HIDDevice oDev)
            : base(oDev)
        {
            m_TrameHID = new byte[oDev.InputReportLength];
        }

        public override void ProcessData()
        {
            m_TrameHID = Buffer;
        }

        public byte[] TrameHID
        {
            get
            {
                return m_TrameHID;
            }
        }
    }
    #endregion
}
